% ------------------------------------------------------------------------------
% define a 1-layer, fully connected neural network (nn)
%
% n_input:       number of input nodes
%                (= number of pixels in the image in case of image classification)
% 
% n_hidden:      number of nodes in the hidden layer
%
% n_output:      number of nodes in the output layer 
%                (= number of classes in case of image classification)
%
% learning_rate: e.g. 0.1
%
% ------------------------------------------------------------------------------

function net = nn_init_net(n_input, n_hidden, n_output, learning_rate)
 
  net = struct();

  net.n_input  = n_input;
  net.n_hidden = n_hidden;
  net.n_output = n_output;

  net.learning_rate = learning_rate;

  % weights between input layer and hidden layer -
  % initialized as a matrix (n_hidden x n_input) 
  % with normal distributed values
  net.weight_ih = randn(n_hidden, n_input, 'single') / sqrt(n_hidden);

  % weights between hidden layer and output layer -
  % initialized as a matrix (n_output x n_hidden)
  % with normal distributed values
  net.weight_ho = randn(n_output, n_hidden, 'single') / sqrt(n_output);

end
