% ------------------------------------------------------------------------------
% train the nn using gradient descent
%
% net:       nn as created with nn_init_net()
%
% inputs:    input layer as matrix (1 x n_input) with single values
%
% responses: correct classification as matrix (n_output x 1)
%            of values 0.01 (wrong) or 0.99 (right)
%
% ------------------------------------------------------------------------------

function net = nn_train_net(net, inputs, responses)

  % (n_hidden x 1) = (n_hidden x n_input) * (n_input x 1)
  hidden_inputs = net.weight_ih * inputs';
  % (n_hidden x 1)
  hidden_outputs = nn_sigmoid(hidden_inputs);
  
  % (n_output x 1) = (n_output x n_hidden) * (n_hidden x 1)
  output_inputs = net.weight_ho * hidden_outputs;
  % (n_output x 1)
  output_outputs = nn_sigmoid(output_inputs);
 
  % (n_output x 1) 
  output_errors = responses - output_outputs;
  % (n_hidden x 1) = (n_hidden x n_output) * (n_output x 1)
  hidden_errors = net.weight_ho' * output_errors;
 
  % add to (n_output x n_hidden):
  % scalar * (n_output x 1) * (n_hidden x 1)'
  net.weight_ho = net.weight_ho +                                  ...
      net.learning_rate *                                          ...
      (output_errors .* output_outputs .* (1 - output_outputs)) *  ...
      hidden_outputs';

  % add to (n_hidden x n_input):
  % scalar * (n_hidden x 1) * (1 x n_input)
  net.weight_ih = net.weight_ih +                                  ...
      net.learning_rate *                                          ...
      (hidden_errors .* hidden_outputs .* (1 - hidden_outputs)) *  ...
      inputs;
      
end
